import pandas as pd
import matplotlib.pyplot as plt
from sklearn.naive_bayes import GaussianNB
from sklearn.ensemble import RandomForestClassifier
from sklearn.svm import SVC
from sklearn.metrics import accuracy_score, average_precision_score, precision_recall_curve
from sklearn.preprocessing import StandardScaler, Imputer
from sklearn.model_selection import train_test_split

RESULT_DATA = "./result_data/result_features.csv"

COLUMN_NAMES = ["bytes_median", "packets_median", "dns_reqs",
                "req/flow", "req/un_dst", "dns_req/un_dns_dst", "paired/flows", "bu/pu", "bd/pd", "bu/bd", "rq/usp",
                "dns_rq/usp", "tunneled"]

FILTER_COLUMNS = ['tunneled', "dns_rq/usp"]

NEGATIVE_SIZE = 100000
POSITIVE_SIZE = 100

NONE_VALUE = -1

# Load Data
data = pd.read_csv(RESULT_DATA, names=COLUMN_NAMES, delimiter=",")
neg_sample = data[data['tunneled'] == 0].sample(NEGATIVE_SIZE, random_state=42).reset_index(drop=True)
pos_sample = data[data['tunneled'] == 1].sample(POSITIVE_SIZE, random_state=42).reset_index(drop=True)
data = pd.concat([pos_sample, neg_sample]).reset_index(drop=True)
labels = data['tunneled']
data = data.drop(columns=FILTER_COLUMNS)

# Missing values
imputer = Imputer(missing_values=NONE_VALUE, strategy='median', axis=0)
imputer = imputer.fit(data)
data = imputer.transform(data)

# Split data
X_train, X_test, y_train, y_test = train_test_split(data, labels, test_size=0.33, random_state=42)

# Feature scaling
sc_X = StandardScaler()
X_train = sc_X.fit_transform(X_train)
X_test = sc_X.transform(X_test)

##### Classifier
# classifier = GaussianNB()
classifier = RandomForestClassifier(random_state=42)
# classifier = SVC(random_state=0, probability=True)
classifier.fit(X_train, y_train)

# Evaluation

# y_score = classifier.decision_function(X_test)
y_score = classifier.predict_proba(X_test)[:, 1]

avg_precision = average_precision_score(y_test, y_score)
print('Average precision-recall score: {0:0.2f}'.format(avg_precision))

precision, recall, _ = precision_recall_curve(y_test, y_score, pos_label=1)

plt.step(recall, precision, color='#005eb8', alpha=0.8, where='mid')
plt.fill_between(recall, precision, step='mid', alpha=0.3, color='#005eb8')

plt.xlabel('Recall')
plt.ylabel("Precision")
plt.ylim([0.0, 1.0])
plt.xlim([0.0, 1.0])
plt.title("Support Vector Machine classifier 1:100 - without Source Port feature\n Precision-Recall curve: AP={0:0.2f}".format(avg_precision))
plt.savefig("SVM1_100_wp.pdf", format='pdf')
plt.show()

