import os
import numpy as np
import pandas as pd

from src.resolver.Helpers import AddressManager

BLANK = "-"
NONE_VALUE = -1000

MIXED_DATA = "./../../../mixed_users/"

COLUMN_NAMES = ['duration', 'proto', 'req_timestamp', 'req_src', 'req_ip', 'req_port', 'req_dst',
                'req_packets', 'req_bytes', 'resp_timestamp', 'resp_src', 'resp_ip', 'resp_port', 'resp_dst',
                'resp_packets', 'resp_bytes', 'paired', 'tunneled']


class DataAnalyser:
    def __init__(self):
        self.result_labels = []
        self.current_frame = pd.DataFrame()

        self.am = AddressManager()

        self.requests = None
        self.dns_requests = None
        self.unique_dst = None
        self.unique_dns_dst = None
        self.flows = None
        self.current_dns_only_paired = None
        self.tunneled = None
        self.unq_src_port = None
        self.dns_unq_src_port = None
        self.dns_current_frame = None

        self.start_feature_function()

    def start_feature_function(self):
        self.feature_function(
            self.get_bytes_median,
            self.get_packets_median,
            self.number_of_dns_requests,
            self.requests_per_flows,
            self.requests_per_unique_dst,
            self.dns_requests_per_unique_dns_dst,
            self.paired_vs_flows_and_up_down_bytes,
            self.requests_per_unique_src_port,
            self.dns_requests_per_unique_src_port
        )

    def feature_function(self, *functions):
        result_list = []
        users = []
        files = os.listdir(MIXED_DATA)
        for file in files:
            self.current_frame = pd.read_csv(MIXED_DATA + file, names=COLUMN_NAMES, delimiter=",",
                                             skiprows=[0])
            self.calc_general_data()
            temp_list = []
            for f in functions:
                # Call all passed functions
                result = f()

                if type(result) is list:
                    temp_list = temp_list + result
                else:
                    temp_list.append(result)
            temp_list.append(self.tunneled)
            result_list.append(temp_list)
            users.append(file.split(".csv")[0])

        result_frame = pd.DataFrame(result_list)
        result_frame.to_csv("./result_data/result_features.csv", index=False, header=False)

        users_frame = pd.DataFrame(users)
        users_frame.to_csv("./result_data/result_users.csv", index=False, header=False)

    def create_label(self):
        label = 0
        if any(self.current_frame['tunneled'] == True):
            label = 1
        return label

    #################################################################################################

    def calc_general_data(self):
        # User tunneled
        self.tunneled = self.create_label()

        # all flows
        self.flows = len(self.current_frame)

        # all requests
        self.requests = self.current_frame['req_packets'].sum()

        # dns_requests
        self.dns_current_frame = self.current_frame[self.current_frame["req_dst"].str.endswith("53")]
        self.dns_requests = self.dns_current_frame['req_packets'].sum()

        # unique dns_dst
        dns_ip_frame = (self.dns_current_frame['req_dst']).drop_duplicates()
        self.unique_dns_dst = len(dns_ip_frame)

        # unique dns src_port
        dns_port_frame = (self.dns_current_frame['req_port']).drop_duplicates()
        self.dns_unq_src_port = len(dns_port_frame)

        # unique_dst
        dst_ip_frame = self.current_frame['req_dst']
        dst_ip_frame = dst_ip_frame.drop_duplicates()
        self.unique_dst = len(dst_ip_frame)

        # Unique src_port
        self.unq_src_port = len((self.current_frame['req_port']).drop_duplicates())

    def get_bytes_median(self):
        bytes_median = self.current_frame['req_bytes'].median()
        return bytes_median

    def get_packets_median(self):
        packets_median = self.current_frame['req_packets'].median()
        return packets_median

    def number_of_dns_requests(self):
        return self.dns_requests

    def requests_per_flows(self):
        if self.flows == 0:
            return NONE_VALUE
        return self.requests / self.flows

    def requests_per_unique_dst(self):
        if self.unique_dst == 0:
            return NONE_VALUE
        return self.requests / self.unique_dst

    def dns_requests_per_unique_dns_dst(self):
        if self.unique_dns_dst == 0:
            return NONE_VALUE
        return self.dns_requests / self.unique_dns_dst

    def paired_per_flows(self):
        paired = len(self.current_dns_only_paired)
        flows = len(self.dns_current_frame)
        if flows == 0:
            return NONE_VALUE
        return paired / flows

    def bytes_up_per_packets_up(self):
        bytes_up = self.dns_current_frame['req_bytes'].sum()
        packets_up = self.dns_current_frame['req_packets'].sum()
        if packets_up == 0:
            return NONE_VALUE
        return bytes_up / packets_up

    def bytes_down_per_packets_down(self):
        if len(self.current_dns_only_paired) == 0:
            return NONE_VALUE
        bytes_down = self.current_dns_only_paired['resp_bytes'].sum()
        packets_down = self.current_dns_only_paired['resp_packets'].sum()
        return bytes_down / packets_down

    def bytes_up_per_bytes_down(self):
        if len(self.current_dns_only_paired) == 0:
            return NONE_VALUE
        bytes_up = self.current_dns_only_paired['req_bytes'].sum()
        bytes_down = self.current_dns_only_paired['resp_bytes'].sum()

        return bytes_up / bytes_down

    def paired_vs_flows_and_up_down_bytes(self):
        self.current_dns_only_paired = self.dns_current_frame[self.dns_current_frame['paired'] == True]
        # self.current_only_paired = self.current_frame[self.current_frame['paired'] == True]
        self.current_dns_only_paired.loc[:, "resp_packets"] = self.current_dns_only_paired.loc[:, "resp_packets"].astype(
            np.float64)
        self.current_dns_only_paired.loc[:, 'resp_bytes'] = self.current_dns_only_paired.loc[:, "resp_bytes"].astype(np.float64)
        result_array = [self.paired_per_flows(),
                        self.bytes_up_per_packets_up(),
                        self.bytes_down_per_packets_down(),
                        self.bytes_up_per_bytes_down()
                        ]
        return result_array

    def requests_per_unique_src_port(self):
        if self.unq_src_port == 0:
            return NONE_VALUE
        return self.requests / self.unq_src_port

    def dns_requests_per_unique_src_port(self):
        if self.dns_unq_src_port == 0:
            return NONE_VALUE
        return self.dns_requests / self.dns_unq_src_port


if __name__ == '__main__':
    da = DataAnalyser()
