import datetime
import itertools

import pandas as pd

from src.resolver.DataLoader import DataLoader
from src.resolver.Helpers import AddressManager
from src.resolver.RecordManager import RecordManager

DNS_PORT = ":53"

BLANK = "-"


class PairManager(DataLoader):
    def __init__(self, input_data):
        super(PairManager, self).__init__(input_data)
        self.data_dict = {}
        self.result_list = []
        self.paired = False
        self.rm = RecordManager()
        self.label_addresses = None

    def create_pair(self, row):
        new_pair = None
        if row.dst_port == DNS_PORT or self.is_address_local(row.src_ip):
            new_pair = self.rm.create_pair_request(row=row)
            if self.label_addresses is not None and row.src_ip in self.label_addresses:
                new_pair = self.rm.label_record(new_pair)
        elif row.src_port == DNS_PORT or not self.is_address_local(row.src_ip):
            new_pair = self.rm.create_pair_response(row=row)
        return new_pair

    def create_map_key(self, src, dst):
        return src + "/" + dst

    def is_packet_relevant(self, new, existing_record):
        if new.proto != existing_record['proto']:
            return False

        ex_is_request = False
        if existing_record["resp_src"] == BLANK and existing_record["resp_dst"] == BLANK \
                and existing_record["req_src"] != BLANK and existing_record["req_dst"] != BLANK:
            ex_is_request = True

        if ex_is_request:
            timestamp_relevant = existing_record["req_timestamp"] <= new.timestamp <= (
                    existing_record["req_timestamp"] + datetime.timedelta(seconds=1))

            is_not_paired = existing_record["paired"] == False
            if is_not_paired and new.src == existing_record["req_dst"] and new.dst == existing_record[
                "req_src"] and timestamp_relevant:
                return True
        else:
            timestamp_relevant = new.timestamp <= (
                    existing_record["resp_timestamp"] + datetime.timedelta(seconds=1))
            is_not_paired = existing_record["paired"] == False
            if is_not_paired and new.src == existing_record["resp_dst"] and new.dst == existing_record[
                "resp_src"] and timestamp_relevant:
                return True

        return False

    def is_address_local(self, address):
        return AddressManager().is_my_address_local(address)

    def update_existing_record(self, row, src, dst):
        data_list = self.data_dict[self.create_map_key(src, dst)]
        paired = False

        if len(data_list) > 0:
            for existing_record in data_list:
                if self.is_packet_relevant(row, existing_record):
                    if existing_record['resp_src'] != BLANK and existing_record['req_src'] == BLANK:
                        existing_record = self.rm.update_pair_with_request(existing_record, row)
                        if self.label_addresses is not None and existing_record["req_ip"] in self.label_addresses:
                            self.rm.label_record(existing_record)
                    elif existing_record['req_src'] != BLANK and existing_record['resp_src'] == BLANK:
                        self.rm.update_pair_with_response(existing_record, row)
                    paired = True
                    break

        if not paired:
            new_pair = self.create_pair(row)
            if new_pair is not None:
                data_list.append(new_pair)

    def insert_new_record(self, row):
        new_pair = self.create_pair(row)
        if new_pair is not None:
            self.data_dict[self.create_map_key(row.src, row.dst)] = []
            self.data_dict[self.create_map_key(row.src, row.dst)].append(new_pair)

    def pair_records(self):
        df = self.load_data_to_dataframe()

        for row in df.itertuples():
            if self.create_map_key(row.src, row.dst) in self.data_dict:
                self.update_existing_record(row, row.src, row.dst)
            elif self.create_map_key(row.dst, row.src) in self.data_dict:
                self.update_existing_record(row, row.dst, row.src)
            else:
                self.insert_new_record(row)

        self.result_list = self.data_dict.values()
        self.result_list = list(itertools.chain.from_iterable(self.result_list))

        self.paired = True


    def get_paired_dataframe(self):
        if not self.paired:
            self.pair_records()

        result_dataframe = pd.DataFrame(self.result_list)
        return result_dataframe

    def get_paired_list(self, label_addresses=None):
        self.label_addresses = label_addresses
        if not self.paired:
            self.pair_records()

        return self.result_list
